#include "asio_output.h"

static void dsd_bufferSwitch(long doubleBufferIndex, ASIOBool directProcess) {
	g_asio_output.get_data(doubleBufferIndex);
}

static void dsd_sampleRateDidChange(ASIOSampleRate sRate) {
}

static long dsd_asioMessage(long selector, long value, void* message, double* opt) {
	switch (selector) {
	case kAsioSelectorSupported:
		switch (value) {
		case kAsioEngineVersion:
		case kAsioResetRequest:
		case kAsioBufferSizeChange:
		case kAsioResyncRequest:
		case kAsioLatenciesChanged:
		case kAsioSupportsTimeInfo:
		case kAsioSupportsTimeCode:
			return 1;
			break;
		default:
			return 0;
			break;
		}
		break;
	case kAsioEngineVersion:
		return 2;
		break;
	case kAsioResetRequest:
		return 1;
		break;
	case kAsioBufferSizeChange:
		return 0;
		break;
	case kAsioResyncRequest:
		return 0;
		break;
	case kAsioLatenciesChanged:
		return 0;
		break;
	case kAsioSupportsTimeInfo:
		return 0;
		break;
	case kAsioSupportsTimeCode:
		return 0;
		break;
	}
	return 0;
}

static ASIOTime* dsd_bufferSwitchTimeInfo(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) {
	return NULL;
}

static HRESULT ole32_CoCreateInstance(REFCLSID rclsid, LPUNKNOWN lpUnkOuter, DWORD dwClsContext, REFIID riid, LPVOID* ppv) {
	HRESULT hr;
	hr = CoCreateInstance(rclsid, lpUnkOuter, dwClsContext, riid, ppv);
	if (SUCCEEDED(hr)) {
		g_proxy.set_CLSID(rclsid);
		g_proxy.set_IASIO((IASIO*)*ppv);
		*ppv = &g_proxy;
	}
	return hr;
}

asio_output_t::asio_output_t() {
	dsd_callbacks.bufferSwitch = dsd_bufferSwitch;
	dsd_callbacks.sampleRateDidChange = dsd_sampleRateDidChange; 
	dsd_callbacks.asioMessage = dsd_asioMessage;
	dsd_callbacks.bufferSwitchTimeInfo = dsd_bufferSwitchTimeInfo; 
	p_put = p_get = new asio_block_t;
	set_cached_blocks(0);
	set_flush_blocks(0);
}

asio_output_t::~asio_output_t() {
}

bool asio_output_t::create(int channels, int samplerate) {
	g_proxy.register_callbacks(&dsd_callbacks);
	return true;
}

bool asio_output_t::dispose() {
	g_proxy.unregister_callbacks();
	return true;
}

void asio_output_t::flush_data() {
	int blocks = get_cached_blocks();
	for (int i = 0; i < blocks; i++) {
		asio_block_t* p_next = p_get->next;
		free(p_get->data);
		delete p_get;
		p_get = p_next;
		decrement_cached_blocks();
	}
}

void asio_output_t::get_data(long doubleBufferIndex) {
	/*
	static int prev_blocks = -1;
	int curr_blocks = cached_blocks;
	if (prev_blocks != curr_blocks) {
		console::printf("cached blocks = %d", curr_blocks);
		prev_blocks = curr_blocks;
	}
	*/
	long dsd_channels = g_proxy.numChannels;
	long dsd_size = g_proxy.dsd_bufferSize * g_proxy.numChannels;
	long dsd_offset = 0;
	while (dsd_size > 0) {
		if (get_cached_blocks() > 0) {
			while (get_cached_blocks() > 1 && get_flush_blocks() > 0) {
				asio_block_t* p_next = p_get->next;
				free(p_get->data);
				delete p_get;
				p_get = p_next;
				decrement_cached_blocks();
				decrement_flush_blocks();
			}
			long dsd_chunk = min(dsd_size, p_get->size - p_get->start);
			for (long i = 0; i < dsd_chunk / dsd_channels; i++) {
				for (long ch = 0; ch < dsd_channels; ch++) {
					((uint8_t*)g_proxy.dsd_buffers[ch].buffers[doubleBufferIndex])[dsd_offset + i] = *(p_get->data + p_get->start + i * dsd_channels + g_proxy.dsd_buffers[ch].channelNum);
				}
			}
			dsd_offset += dsd_chunk / dsd_channels;
			dsd_size -= dsd_chunk;
			p_get->start += dsd_chunk;
			if (p_get->start >= p_get->size) {
				asio_block_t* p_next = p_get->next;
				free(p_get->data);
				delete p_get;
				p_get = p_next;
				decrement_cached_blocks();
			}
		}
		else {
			for (long ch = 0; ch < dsd_channels; ch++) {
				memset(g_proxy.dsd_buffers[ch].buffers[doubleBufferIndex], 0x55, g_proxy.dsd_bufferSize);
			}
		}
	}
}

bool asio_output_t::set_data(uint8_t* dsd_data, size_t dsd_size) {
	if (p_put) {
		p_put->data = (uint8_t*)malloc(dsd_size);
		p_put->size = dsd_size;
		memcpy(p_put->data, dsd_data, p_put->size);
		p_put->start = 0;
		p_put->next = new asio_block_t;
		p_put = p_put->next;
		increment_cached_blocks();
		return true;
	}
	return false;
}

void asio_output_t::playback_start() {
}

void asio_output_t::playback_stop() {
}

void asio_output_t::playback_seek() {
	request_flush();
}

void asio_output_t::playback_pause(bool playback_state) {
	if (playback_state) {
	}
	else {
	}
}

bool asio_output_t::hook(bool install) {
	hAsioOutput = GetModuleHandle(_T("foo_out_asio"));
	if (hAsioOutput) {
		HMODULE hOle32 = GetModuleHandle(_T("ole32"));
		if (hOle32) {
			fpCoCreateInstance = (CoCreateInstance_t*)GetProcAddress(hOle32, "CoCreateInstance");
			if (fpCoCreateInstance) {
				PEImage img;
				if (img.map(hAsioOutput)) {
					if (install) {
						return img.install_thunk("ole32.dll", (FARPROC)fpCoCreateInstance, (FARPROC)ole32_CoCreateInstance);
					}
					else {
						return img.install_thunk("ole32.dll", (FARPROC)ole32_CoCreateInstance, (FARPROC)fpCoCreateInstance);
					}
				}
			}
		}
	}
	return false;
}

/*
* PEImage Class
*/

bool PEImage::map(HMODULE hMod) {
	int f_pos = 0;
	p_mod = (PBYTE)hMod;
	p_idh = (PIMAGE_DOS_HEADER)&p_mod[f_pos];
	if (memcmp(&p_idh->e_magic, "MZ", 2)) return false; // check DOS signature
	f_pos = p_idh->e_lfanew; // get PE section
	if (memcmp(&p_mod[f_pos], "PE\0\0", 4)) return false; // check PE signature
	f_pos += 4;
	p_ifh = (PIMAGE_FILE_HEADER)&p_mod[f_pos];
	f_pos += IMAGE_SIZEOF_FILE_HEADER;
	p_ioh = (PIMAGE_OPTIONAL_HEADER)&p_mod[f_pos];
	f_pos += p_ifh->SizeOfOptionalHeader;
	p_ish = (PIMAGE_SECTION_HEADER)&p_mod[f_pos];
	f_pos += IMAGE_SIZEOF_SECTION_HEADER * p_ifh->NumberOfSections;
	f_pos = p_ioh->DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT].VirtualAddress;
	p_iid = (PIMAGE_IMPORT_DESCRIPTOR)&p_mod[f_pos];
	return true;
}

bool PEImage::install_thunk(LPSTR lpszLibName, FARPROC fpOldProc, FARPROC fpNewProc) {
	for (int i = 0; ; i++) {
		if (!p_iid[i].Characteristics && !p_iid[i].TimeDateStamp && !p_iid[i].ForwarderChain && !p_iid[i].Name && !p_iid[i].FirstThunk) {
			break;
		}
		LPSTR lpszLib = (LPSTR)(p_mod + p_iid[i].Name);
		if (strcmp(lpszLib, lpszLibName) == 0) {
			PIMAGE_THUNK_DATA p_itd = (PIMAGE_THUNK_DATA)(p_mod + p_iid[i].FirstThunk);
			for (int j = 0; ; j++) {
				if (!p_itd[j].u1.Function) {
					break;
				}
				if (p_itd[j].u1.Function == (DWORD)fpOldProc) {
					DWORD flOldProtect;
					if (VirtualProtect(&p_itd[j].u1.Function, sizeof(DWORD), PAGE_READWRITE, &flOldProtect)) {
						p_itd[j].u1.Function = (DWORD)fpNewProc;
						VirtualProtect(&p_itd[j].u1.Function, sizeof(DWORD), flOldProtect, &flOldProtect);
						VirtualLock(&p_itd[j].u1.Function, sizeof(DWORD));
						return true;
					}
				}
			}
		}
	}
	return false;
}

/*
* ASIO Proxy Class
*/

ASIOCallbacks   CoASIOProxy::dsd_callbacks;
ASIOCallbacks   CoASIOProxy::pcm_callbacks;
ASIOCallbacks   CoASIOProxy::ext_callbacks;
ASIOChannelInfo CoASIOProxy::channels[ASIO_MAXCHANNELS];
ASIOBufferInfo  CoASIOProxy::dsd_buffers[ASIO_MAXCHANNELS];
ASIOBufferInfo  CoASIOProxy::pcm_buffers[ASIO_MAXCHANNELS];
asio_mode_t     CoASIOProxy::mode;
CLSID           CoASIOProxy::clsid;
IASIO*          CoASIOProxy::pIASIO;
long            CoASIOProxy::numChannels;
long            CoASIOProxy::dsd_bufferSize;
long            CoASIOProxy::pcm_bufferSize;
long            CoASIOProxy::dsd_sampleSize[ASIO_MAXCHANNELS];
long            CoASIOProxy::pcm_sampleSize[ASIO_MAXCHANNELS];
double          CoASIOProxy::dsd_sampleRate;
double          CoASIOProxy::pcm_sampleRate;
asio_mode_t     CoASIOProxy::requested_mode;
double          CoASIOProxy::requested_dsd_sampleRate;
double          CoASIOProxy::requested_pcm_sampleRate;

void CoASIOProxy::bufferSwitch(long doubleBufferIndex, ASIOBool directProcess) {
	if (pcm_callbacks.bufferSwitch) {
		pcm_callbacks.bufferSwitch(doubleBufferIndex, directProcess);
	}
	if (ext_callbacks.bufferSwitch) {
		ext_callbacks.bufferSwitch(doubleBufferIndex, directProcess);
	}
}

void CoASIOProxy::sampleRateDidChange(ASIOSampleRate sampleRate) {
	dsd_sampleRate = sampleRate;
	if (pcm_callbacks.sampleRateDidChange) {
		pcm_callbacks.sampleRateDidChange(pcm_sampleRate);
	}
	if (ext_callbacks.sampleRateDidChange) {
		ext_callbacks.sampleRateDidChange(pcm_sampleRate);
	}
}

long CoASIOProxy::asioMessage(long selector, long value, void* message, double* opt) {
	long rv = 0;
	switch (selector) {
	case kAsioBufferSizeChange:
		dsd_bufferSize = value;
		pcm_bufferSize = getPcmBufSize(value);
		break;
	case kAsioResyncRequest:
		break;
	case kAsioLatenciesChanged:
		break;
	}
	if (pcm_callbacks.asioMessage) {
		rv = pcm_callbacks.asioMessage(selector, value, message, opt);
	}
	if (ext_callbacks.asioMessage) {
		rv = ext_callbacks.asioMessage(selector, value, message, opt);
	}
	return rv;
}

ASIOTime* CoASIOProxy::bufferSwitchTimeInfo(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) {
	ASIOTime* pTime;
	if (pcm_callbacks.bufferSwitchTimeInfo) {
		pTime = pcm_callbacks.bufferSwitchTimeInfo(params, doubleBufferIndex, directProcess);
	}
	if (ext_callbacks.bufferSwitchTimeInfo) {
		pTime = ext_callbacks.bufferSwitchTimeInfo(params, doubleBufferIndex, directProcess);
	}
	return pTime;
}

CoASIOProxy::CoASIOProxy() {
	dsd_callbacks.bufferSwitch         = bufferSwitch;
	dsd_callbacks.sampleRateDidChange  = sampleRateDidChange;
	dsd_callbacks.asioMessage          = asioMessage;
	dsd_callbacks.bufferSwitchTimeInfo = bufferSwitchTimeInfo;
	memset(&pcm_callbacks, 0, sizeof(ASIOCallbacks));
	memset(&ext_callbacks, 0, sizeof(ASIOCallbacks));
	mode           = ASIO_MODE_PCM;
	clsid          = GUID_NULL;
	pIASIO         = NULL;
	numChannels    = 0;
	dsd_bufferSize = 0;
	pcm_bufferSize = 0;
	dsd_sampleRate = DSDxFs64;
	pcm_sampleRate = DSDxFs1;
	requested_mode = mode;
}

CoASIOProxy::~CoASIOProxy() {
}

/*
* IUnknown Interface
*/

STDMETHODIMP CoASIOProxy::QueryInterface(REFIID riid, LPVOID* ppv) {
	HRESULT hr;
	hr = pIASIO->QueryInterface(riid, ppv);
	return hr;
}

STDMETHODIMP_(ULONG) CoASIOProxy::AddRef() {
	ULONG refs;
	refs = pIASIO->AddRef();
	return refs;
}

STDMETHODIMP_(ULONG) CoASIOProxy::Release() {
	ULONG refs;
	refs = pIASIO->Release();
	return refs;
}

/*
* IASIO Interface
*/

ASIOBool CoASIOProxy::init(void* sysHandle) {
	ASIOError rc;
	rc = pIASIO->init(sysHandle);
	set_mode(requested_mode);
	return rc;
}

void CoASIOProxy::getDriverName(char* name) {
	pIASIO->getDriverName(name);
}

long CoASIOProxy::getDriverVersion() {
	long version;
	version =	pIASIO->getDriverVersion();
	return version;
}

void CoASIOProxy::getErrorMessage(char* string) {
	pIASIO->getErrorMessage(string);
}

ASIOError CoASIOProxy::start() {
	ASIOError rc;
	set_mode(requested_mode);
	rc = pIASIO->start();
	return rc;
}

ASIOError CoASIOProxy::stop() {
	ASIOError rc;
	rc = pIASIO->stop();
	set_mode(requested_mode);
	return rc;
}

ASIOError CoASIOProxy::getChannels(long* numInputChannels, long* numOutputChannels) {
	ASIOError rc;
	set_mode(requested_mode);
	rc = pIASIO->getChannels(numInputChannels, numOutputChannels);
	return rc;
}

ASIOError CoASIOProxy::getLatencies(long* inputLatency, long* outputLatency) {
	ASIOError rc;
	set_mode(requested_mode);
	rc = pIASIO->getLatencies(inputLatency, outputLatency);
	switch (mode) {
	case ASIO_MODE_DSD:
		*inputLatency = getPcmBufSize(*inputLatency);
		*outputLatency = getPcmBufSize(*outputLatency);
		break;
	case ASIO_MODE_PCM:
		break;
	}
	return rc;
}

ASIOError CoASIOProxy::getBufferSize(long* minSize, long* maxSize,	long* preferredSize, long* granularity) {
	ASIOError rc;
	set_mode(requested_mode);
	rc = pIASIO->getBufferSize(minSize, maxSize, preferredSize, granularity);
	switch (mode) {
	case ASIO_MODE_DSD:
		*minSize = getPcmBufSize(*minSize);
		*maxSize = getPcmBufSize(*maxSize);
		*preferredSize = getPcmBufSize(*preferredSize);
		*granularity = getPcmBufSize(*granularity);
		break;
	case ASIO_MODE_PCM:
		break;
	}
	return rc;
}

ASIOError CoASIOProxy::canSampleRate(ASIOSampleRate sampleRate) {
	ASIOError rc;
	set_mode(requested_mode);
	switch (mode) {
	case ASIO_MODE_DSD:
		if (dsd_sampleRate == DSDxFs64 || dsd_sampleRate == DSDxFs128) {
			rc = pIASIO->canSampleRate(pcm_sampleRate);
		}
		else {
			rc = ASE_NoClock;
		}
		break;
	case ASIO_MODE_PCM:
		rc = pIASIO->canSampleRate(sampleRate);
		break;
	}
	return rc;
}

ASIOError CoASIOProxy::getSampleRate(ASIOSampleRate* sampleRate) {
	ASIOError rc;
	set_mode(requested_mode);
	switch (mode) {
	case ASIO_MODE_DSD:
		*sampleRate = pcm_sampleRate;
		rc = ASE_OK;
		break;
	case ASIO_MODE_PCM:
		rc = pIASIO->getSampleRate(sampleRate);
		break;
	}
	return rc;
}

ASIOError CoASIOProxy::setSampleRate(ASIOSampleRate sampleRate) {
	ASIOError rc;
	set_mode(requested_mode);
	switch (mode) {
	case ASIO_MODE_DSD:
		if (sampleRate == DSDxFs1 || sampleRate == DSDxFs2 || sampleRate == DSDxFs4) {
			rc = pIASIO->setSampleRate(dsd_sampleRate);
		}
		else {
			rc = ASE_NoClock;
		}
		break;
	case ASIO_MODE_PCM:
		rc = pIASIO->setSampleRate(sampleRate);
		break;
	}
	return rc;
}

ASIOError CoASIOProxy::getClockSources(ASIOClockSource* clocks, long* numSources) {
	ASIOError rc;
	set_mode(requested_mode);
	rc = pIASIO->getClockSources(clocks, numSources);
	return rc;
}

ASIOError CoASIOProxy::setClockSource(long index) {
	ASIOError rc;
	set_mode(requested_mode);
	rc = pIASIO->setClockSource(index);
	return rc;
}

ASIOError CoASIOProxy::getSamplePosition(ASIOSamples* sPos, ASIOTimeStamp* tStamp) {
	ASIOError rc;
	rc = pIASIO->getSamplePosition(sPos, tStamp);
	switch (mode) {
	case ASIO_MODE_DSD:
		*(uint64_t*)sPos = (uint64_t)getPcmBufSize(64) * (*(uint64_t*)sPos) / 64;
		break;
	case ASIO_MODE_PCM:
		break;
	}
	return rc;
}

ASIOError CoASIOProxy::getChannelInfo(ASIOChannelInfo* info) {
	ASIOError rc;
	set_mode(requested_mode);
	rc = pIASIO->getChannelInfo(info);
	switch (info->type) {
	case ASIOSTDSDInt8LSB1:
	case ASIOSTDSDInt8MSB1:
	case ASIOSTDSDInt8NER8:
		info->type = ASIOSTInt32LSB;
		break;
	}
	return rc;
}

ASIOError CoASIOProxy::createBuffers(ASIOBufferInfo* bufferInfos, long numChannels, long bufferSize, ASIOCallbacks* callbacks) {
	ASIOError rc;
	long dsd_minSize, dsd_maxSize, dsd_preferredSize, dsd_granularity, pcm_maxSize;
	this->numChannels = numChannels;
	memset(&pcm_callbacks, 0, sizeof(ASIOCallbacks));
	switch (mode) {
	case ASIO_MODE_DSD:
		rc = pIASIO->getBufferSize(&dsd_minSize, &dsd_maxSize, &dsd_preferredSize, &dsd_granularity);
		if (rc != ASE_OK) {
			return rc;
		}
		pcm_bufferSize = bufferSize;
		pcm_maxSize = getPcmBufSize(dsd_maxSize);
		dsd_bufferSize = getDsdBufSize(bufferSize);
		for (long ch = 0; ch < numChannels; ch++) {
			channels[ch].channel = bufferInfos[ch].channelNum;
			channels[ch].isInput = bufferInfos[ch].isInput;
			pIASIO->getChannelInfo(&channels[ch]);
			dsd_sampleSize[ch] = getSampleSizeInBytes(ASIOSTDSDInt8MSB1);
			pcm_sampleSize[ch] = getSampleSizeInBytes(ASIOSTInt32LSB);
			dsd_buffers[ch].isInput    = bufferInfos[ch].isInput;
			dsd_buffers[ch].channelNum = bufferInfos[ch].channelNum;
			pcm_buffers[ch].isInput    = bufferInfos[ch].isInput;
			pcm_buffers[ch].channelNum = bufferInfos[ch].channelNum;
			pcm_buffers[ch].buffers[0] = malloc(pcm_maxSize * pcm_sampleSize[ch]);
			pcm_buffers[ch].buffers[1] = malloc(pcm_maxSize * pcm_sampleSize[ch]);
			bufferInfos[ch].buffers[0] = pcm_buffers[ch].buffers[0];
			bufferInfos[ch].buffers[1] = pcm_buffers[ch].buffers[1];
		}
		for (long ch = 0; ch < numChannels; ch++) {
			if (pcm_buffers[ch].buffers[0] == NULL || pcm_buffers[ch].buffers[1] == NULL) {
				return ASE_NoMemory;
			}
		}
		rc = pIASIO->createBuffers(dsd_buffers, numChannels, dsd_bufferSize, &dsd_callbacks);
		if (rc == ASE_OK) {
			memcpy(&pcm_callbacks, callbacks, sizeof(ASIOCallbacks));
		}
		break;	
	case ASIO_MODE_PCM:
		for (long ch = 0; ch < numChannels; ch++) {
			dsd_buffers[ch].buffers[0] = NULL;
			dsd_buffers[ch].buffers[1] = NULL;
			pcm_buffers[ch].buffers[0] = NULL;
			pcm_buffers[ch].buffers[1] = NULL;
		}
		rc = pIASIO->createBuffers(bufferInfos, numChannels, bufferSize, callbacks);
		break;
	}
	return rc;
}

ASIOError CoASIOProxy::disposeBuffers() {
	ASIOError rc;
	rc = pIASIO->disposeBuffers();
	memset(&pcm_callbacks, 0, sizeof(ASIOCallbacks));
	for (long ch = 0; ch < numChannels; ch++) {
		if (pcm_buffers[ch].buffers[0]) {
			free(pcm_buffers[ch].buffers[0]);
			pcm_buffers[ch].buffers[0] = NULL;
		}
		if (pcm_buffers[ch].buffers[1]) {
			free(pcm_buffers[ch].buffers[1]);
			pcm_buffers[ch].buffers[1] = NULL;
		}
	}
	numChannels = 0;
	dsd_bufferSize = 0;
	pcm_bufferSize = 0;
	return rc;
}

ASIOError CoASIOProxy::controlPanel() {
	ASIOError rc;
	rc = pIASIO->controlPanel();
	return rc;
}

ASIOError CoASIOProxy::future(long selector, void* opt) {
	ASIOError rc;
	rc = pIASIO->future(selector, opt);
	return rc;
}

ASIOError CoASIOProxy::outputReady() {
	ASIOError rc;
	rc = pIASIO->outputReady();
	return rc;
}

/*
* Privates
*/

ASIOError CoASIOProxy::set_mode(asio_mode_t new_mode) {
	ASIOError rc = ASE_SUCCESS;
	ASIOIoFormat format;
	switch (new_mode) {
	case ASIO_MODE_DSD:
		format.FormatType = kASIODSDFormat;
		break;
	case ASIO_MODE_PCM:
		format.FormatType = kASIOPCMFormat;
		break;
	default:
		format.FormatType = kASIOFormatInvalid;
		break;
	}
	rc = pIASIO->future(kAsioSetIoFormat, &format);
	if (rc == ASE_SUCCESS) {
		mode = new_mode;
		dsd_sampleRate = requested_dsd_sampleRate;
		pcm_sampleRate = requested_pcm_sampleRate;
	}
	return rc;
}

long CoASIOProxy::getSampleSizeInBytes(ASIOSampleType type) {
	long size;
	switch (type) {
	case ASIOSTDSDInt8LSB1:
	case ASIOSTDSDInt8MSB1:
	case ASIOSTDSDInt8NER8:
		size = 1;
		break;
	case ASIOSTInt16MSB:
	case ASIOSTInt16LSB:
		size = 2;
		break;
	case ASIOSTInt24MSB:
	case ASIOSTInt24LSB:
		size = 3;
		break;
	case ASIOSTInt32MSB:
	case ASIOSTInt32MSB16:
	case ASIOSTInt32MSB18:
	case ASIOSTInt32MSB20:
	case ASIOSTInt32MSB24:
	case ASIOSTFloat32MSB:
	case ASIOSTInt32LSB:
	case ASIOSTInt32LSB16:
	case ASIOSTInt32LSB18:
	case ASIOSTInt32LSB20:
	case ASIOSTInt32LSB24:
	case ASIOSTFloat32LSB:
		size = 4;
		break;
	case ASIOSTFloat64MSB:
		size = 8;
		break;
	default:
		size = 0;
		break;
	}
	return size;
}

long CoASIOProxy::getDsdBufSize(long pcm_bufSize) {
	long dsd_bufSize;
	dsd_bufSize = (long)(pcm_bufSize * (dsd_sampleRate / 8 / pcm_sampleRate));
	return dsd_bufSize;
}

long CoASIOProxy::getPcmBufSize(long dsd_bufSize) {
	long pcm_bufSize;
	pcm_bufSize = (long)(dsd_bufSize / (dsd_sampleRate / 8 / pcm_sampleRate));
	return pcm_bufSize;
}
