/*
* SACD Decoder plugin
* Copyright (c) 2011 Maxim V.Anisiutkin <maxim.anisiutkin@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with FFmpeg; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#ifndef _SACD_DSDIFF_H_INCLUDED
#define _SACD_DSDIFF_H_INCLUDED

#include <stdint.h>
#include <foobar2000.h>
#include "endianess.h"
#include "scarletbook.h"
#include "sacd_reader.h"

#pragma pack(1)

class ID {
public:
	uint8_t ckID[4];
	bool inline has_id(const char* id) {
		return ckID[0] == id[0] && ckID[1] == id[1] && ckID[2] == id[2] && ckID[3] == id[3];
	}
};

class Chunk : public ID {
public:
	uint64_t ckDataSize;
	uint64_t inline get_size() {
		return hton64(ckDataSize);
	}
};

class FormDSDChunk : public Chunk {
public:
	ID formType;
};

#pragma pack()

class sacd_dsdiff_t : public sacd_reader_t {
	abort_callback_impl m_abort;
public:
	service_ptr_t<file> m_file;
	uint32_t m_version;
	uint32_t m_samplerate;
	uint16_t m_channel_count;
	int      m_dst_encoded;
	uint64_t m_data_offset;
	uint64_t m_data_size;
	uint16_t m_framerate;
	uint32_t m_frame_size;
	uint32_t m_frame_count;
public:
	sacd_dsdiff_t(void);
	~sacd_dsdiff_t(void);
	int get_channels();
	int get_samplerate();
	uint64_t get_size();
	uint64_t get_offset();
	double get_duration();
	bool is_dst();
	bool open(service_ptr_t<file> p_file);
	bool close();
	bool read_frame(uint8_t* frame_data, size_t* frame_size);
	bool seek(uint64_t offset);
};

#endif
