/*
* SACD Decoder plugin
* Copyright (c) 2011 Maxim V.Anisiutkin <maxim.anisiutkin@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with FFmpeg; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#ifndef _DSD2PCM_CONV_H_INCLUDED
#define _DSD2PCM_CONV_H_INCLUDED

#include <stdint.h>
#include "conststr.h"

#define FIFOSIZE 64
#define FIFOMASK (FIFOSIZE - 1)

typedef float real_t;

typedef struct {
	uint8_t fifo[FIFOSIZE];
	size_t  fifopos;
} dsd2pcm_channel_t;

typedef struct {
	int               channels;
	int               dsd_samplerate;
	int               pcm_samplerate;
	int               decimation;
	real_t            gain;
	dsd2pcm_channel_t ch[MAX_CHANNELS];
} dsd2pcm_converter_t;

#ifdef __cplusplus
extern "C"
{
#endif

void dsd2pcm_converter_init(dsd2pcm_converter_t* dsd2pcm, int channels, int dsd_samplerate, int pcm_samplerate);
void dsd2pcm_converter_convert(dsd2pcm_converter_t* dsd2pcm, uint8_t* dsd_data, real_t* pcm_data);
void dsd2pcm_converter_set_gain(dsd2pcm_converter_t* dsd2pcm, real_t dB_gain);
void dsd2pcm_converter_save(dsd2pcm_converter_t* dsd2pcm);

#ifdef __cplusplus
}
#endif

#endif
