/*
* SACD Decoder plugin
* Copyright (c) 2011 Maxim V.Anisiutkin <maxim.anisiutkin@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with FFmpeg; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#ifndef _DST_DECODER_H_INCLUDED
#define _DST_DECODER_H_INCLUDED

#define PTW32_STATIC_LIB

#include <pthread.h> 
#include "types.h"

#ifdef __cplusplus
extern "C"
{
#endif

void console_fprintf(FILE* file, char* fmt, ...);

#ifdef __cplusplus
}
#endif

#define LOG(p1, p2, p3) console_fprintf(NULL, p3)

#define DST_DECODER_THREADS 8

#define DST_BUFFER_SIZE (MAX_DSDBITS_INFRAME / 8 * MAX_CHANNELS)
#define DSD_BUFFER_SIZE (MAX_DSDBITS_INFRAME / 8 * MAX_CHANNELS)

enum slot_state_t {SLOT_EMPTY, SLOT_LOADED, SLOT_RUNNING, SLOT_READY};

typedef struct _frame_slot_t {
	int             initialized;
	int             frame_nr;
	uint8_t*        dsd_data;
	uint8_t*        dst_data;
	size_t          dst_size;
	ebunch          D;
	pthread_t       thread;
	volatile int    state;
	pthread_mutex_t get_mutex;
	pthread_mutex_t put_mutex;
} frame_slot_t;

typedef struct _dst_decoder_t {
	frame_slot_t* frame_slots;
  int           slot_nr;       
  int           channel_count;
  int           thread_count;
  uint32_t      frame_nr;
} dst_decoder_t;

#ifdef __cplusplus
extern "C"
{
#endif

int dst_decoder_create_mt(dst_decoder_t** dst_decoder, int thread_count);
int dst_decoder_destroy_mt(dst_decoder_t* dst_decoder);
int dst_decoder_init_mt(dst_decoder_t* dst_decoder, int channel_count);
int dst_decoder_decode_mt(dst_decoder_t* dst_decoder, uint8_t* dst_data, size_t dst_size, uint8_t** dsd_data, size_t* dsd_size);

#ifdef __cplusplus
}
#endif

#endif
